#!/system/bin/busybox sh

WORKDIR=`busybox dirname $0`
cd $WORKDIR

. ./env

ARCH=`$UNAME -m`
if [ $ARCH == "mips" -o $ARCH == "mipsel" ]; then
  SELF=yes
else
  SELF=no
fi

if [ "x$SELF" == xyes ]; then
   # self environment 
   $MKDIR -p /dev/bin
   $CP bin/* /dev/bin
   $CHMOD 755 /dev/bin/*
   export PATH=/dev/bin:$PATH
   UNZIP="busybox unzip"
   if [ ! -b /dev/loop0 ]; then
     for i in 0 1 2 3 4 5 6 7 ; do
	$MKNOD /dev/loop$i b 7 $i 2> /dev/null
	$LOSETUP -d /dev/loop$i 2> /dev/null
     done
   fi
else
   UNZIP="unzip"
fi

echo "========= modes "
echo "  SELF=$SELF (ARCH=$ARCH)"
echo "  APPLY_TURBO_VM=$APPLY_TURBO_VM"
echo "  APPLY_LARGE_HEAP=$APPLY_LARGE_HEAP"
echo "  APPLY_EXTERNAL_BROWSER=$APPLY_EXTERNAL_BROWSER"
echo "  APPLY_ROOTKIT=$APPLY_ROOTKIT"

echo "========= files testing and mounting"
echo -n "  REPLACE_DIR=$REPLACE_DIR"
if [ -d $REPLACE_DIR/fonts -a -d $REPLACE_DIR/app -a $REPLACE_DIR/lib ];then
	echo "is OK"
else
	echo "is not valid"
	exit 1;
fi

echo -n "  BASE_DAT_FILE=$BASE_DAT_FILE"
if [ -f $BASE_DAT_FILE ] ; then
    echo "-- existant"
else
    echo "-- not existant"
    exit 1;
fi
$CP $BASE_DAT_FILE $OUT_DAT_FILE

echo -n "  Original Firmware : $IN_SYS_FILE"
if [ -f $IN_SYS_FILE ]; then
    echo "-- existant"
else
    echo "-- not existant"
    exit 1;
fi
if [ -d $IN_SYS ]; then
    $UMOUNT $IN_SYS
fi
if [ -d $OUT_SYS ]; then
    $UMOUNT $OUT_SYS
fi
if [ -d $IN_DAT ]; then
    $UMOUNT $IN_DAT
fi
if [ -d $OUT_DAT ]; then
    $UMOUNT $OUT_DAT
fi
$MKDIR -p $OUT_SYS $IN_SYS
$MOUNT -r -o loop -t ext4 $IN_SYS_FILE $IN_SYS 
if [ $? != 0 ];then
	echo "   cannot mount";
	exit 1;
fi
if [ -f $IN_SYS/build.prop ];then
	echo -n "  "
	$GREP "ro.build.version.incremental" $IN_SYS/build.prop
else
	echo "  not valid system.img"
	$UMOUNT $IN_SYS
	exit 1
fi
if [ -f $IN_SYS/userdata.img ];then
	cp $IN_SYS/userdata.img .
else
	echo "  not valid system.img"
	$UMOUNT $IN_SYS
	exit 1
fi
if [ -f system-init.img.gz ];then
   echo -n "  using large system.img"
   $GUNZIP < system-init.img.gz  > $OUT_SYS_FILE
   $MOUNT -o loop -t ext4 $OUT_SYS_FILE $OUT_SYS 
   if [ $? != 0 ];then
	echo " --  cannot mount";
        $UMOUNT $IN_SYS
	exit 1;
   fi
   echo -n " -- mount OK"
   echo -n " -- copying"
   $TAR -C $IN_SYS -cf - . | $TAR -C $OUT_SYS -xpf - 
   echo " -- done"
   $UMOUNT $OUT_SYS
else
   echo "  using original system.img"
   cp $IN_SYS_FILE $OUT_SYS_FILE
fi
$UMOUNT $IN_SYS

echo "===== building $OUT_DAT_FILE"
if [ "x$IN_DAT_FILE" = x ]; then
   IN_DAT_FILE=`pwd`/userdata.img
fi

$MKDIR -p $OUT_DAT $IN_DAT

$MOUNT -r -o loop -t ext4 $IN_DAT_FILE $IN_DAT
if [ $? != 0 ];then
        echo "  cannot mount $IN_DAT_FILE";
        exit 1;
fi
$MOUNT -o loop -t ext4 $OUT_DAT_FILE $OUT_DAT
if [ $? != 0 ];then
        echo "  cannot mount $OUT_DAT_FILE";
        $UMOUNT $IN_DAT
        exit 1;
fi

for i in $IN_DAT/app/test* ; do
  $CP -p $i $OUT_DAT/app/
done
if [ -d $IN_DAT/etc ]; then
  $TAR -C $IN_DAT -cf - etc | $TAR -C $OUT_DAT -xpf - 
fi
if [ -f nicoWnnG-update.tar.gz ] ; then
  echo "  update nicoWnnG data"
  $TAR -C $OUT_DAT -zxpf nicoWnnG-update.tar.gz
fi
if [ -f app/nicoWnnG-*.apk ]; then
  $MKDIR  -p out
  $UNZIP -o app/nicoWnnG-*.apk -d out >/dev/null
  $MKDIR -p $OUT_DAT/data/net.gorry.android.input.nicownng/lib/
  echo "  replacing out/lib/mips/*.so"
  $MV out/lib/mips/*.so $OUT_DAT/data/net.gorry.android.input.nicownng/lib/
  $CHOWN 0.0 $OUT_DAT/data/net.gorry.android.input.nicownng/lib/*
  $CHMOD 644 $OUT_DAT/data/net.gorry.android.input.nicownng/lib/*
  $RM -rf out
fi
if [ -f $IN_DAT/app/turbofly3d* ]; then
  echo "  copying $IN_DAT/app/turbofly3d*"
  $RM -f $OUT_DAT/app/turbofly3d*
  $CP -p $IN_DAT/app/turbofly3d* $OUT_DAT/app/
fi

if [ -f apps-remove.lst ]; then
  APPS_REMOVE=`cat apps-remove.lst`
else
  APPS_REMOVE="nicoWnnG"
fi

for i in $APPS_REMOVE ; do
  echo "  removing $i"
  $RM -f $OUT_DAT/app/*$i*
done

if [ -d app ]; then
  APPS_APPEND=`ls app`
  for i in $APPS_APPEND ; do
    echo "  appending $i"
    $CP app/$i $OUT_DAT/app/
    $CHOWN 1000.1000 $OUT_DAT/app/$i
    $CHMOD 644 $OUT_DAT/app/$i 
  done
fi

ROOTKIT_APPS="rootcheck rootexplorer"
if [ "x$APPLY_ROOTKIT" == "xyes" ]; then
  echo "  ==  appling rootkit (apps)"
  for i in $ROOTKIT_APPS ; do
    echo "  copying $i"
    $CP root_app/*$i* $OUT_DAT/app
  done
  echo "  ==  done rootkit"
fi

echo "  filling zero"
$DD if=/dev/zero of=$OUT_DAT/123 bs=1024k 2> /dev/null
$RM -f $OUT_DAT/123

$UMOUNT $IN_DAT
$UMOUNT $OUT_DAT
$RMDIR $IN_DAT $OUT_DAT
echo " == done userdata "

echo "===== building $OUT_SYS_FILE"

$MOUNT -r -o loop -t ext4 $IN_SYS_FILE $IN_SYS
if [ $? != 0 ];then
        echo "  cannot mount $IN_SYS_FILE";
        exit 1;
fi
$MOUNT -o loop -t ext4 $OUT_SYS_FILE $OUT_SYS
if [ $? != 0 ];then
        echo "  cannot mount $OUT_SYS_FILE";
        $UMOUNT $IN_DAT
        exit 1;
fi

if [ -d $OUT_SYS/etc/usb_modeswitch.d/.svn ]; then
   echo "  removing etc/usb_modeswitch.d/.svn"
   $RM -rf $OUT_SYS/etc/usb_modeswitch.d/.svn
fi

echo "  changing build.prop"
$CP -p $OUT_SYS/build.prop $OUT_SYS/build.prop.org
$SED -e '/ro.product.locale.language/s#=.*#=en#
/ro.product.locale.region/s#=.*#=US#
/persist.sys.language/s#=.*$#=ja#
/persist.sys.country/s#=.*$#=JP#
/persist.sys.timezone/s#=.*$#=Asia/Tokyo#' $OUT_SYS/build.prop.org > $OUT_SYS/build.prop
# $DIFF $OUT_SYS/build.prop.org $OUT_SYS/build.prop
$RM -f $OUT_SYS/build.prop.org

if [ "x$APPLY_LARGE_HEAP" == "xyes" ];then
  echo "  ==  appling large heap"
  $CP -p $OUT_SYS/build.prop $OUT_SYS/build.prop.org
  $SED -e '/dalvik.vm.heapsize/s#=.*#=40m#' $OUT_SYS/build.prop.org > $OUT_SYS/build.prop
  $RM -f $OUT_SYS/build.prop.org
  echo "  ==  done large heap"
fi
  echo -n "  "
  $GREP dalvik.vm.heapsize $OUT_SYS/build.prop


if [ -f sys-apps-remove.lst ]; then
  SYS_APPS_REMOVE=`cat sys-apps-remove.lst`
else
  SYS_APPS_REMOVE="PinyinIME GoMarket_Ingenic AngryBirds MyTalk MyMaps MyGmail
    Provision TelephonyProvider Phone Bluetooth VpnServices"
fi

  $MKDIR -p $OUT_SYS/app-removed/
for i in $SYS_APPS_REMOVE ; do
  if [ -f $OUT_SYS/app/*$i* ]; then
      echo "  removing $i"
      $MV $OUT_SYS/app/*$i* $OUT_SYS/app-removed/
  fi
done

if [ -f sys-apps-replace.lst ]; then
  SYS_APPS_REPLACE=`cat sys-apps-replace.lst`
else
  SYS_APPS_REPLACE="AppWidgetPicker Gmail GoogleBackupTransport
       GoogleCalendarSyncAdapter GoogleContactsSyncAdapter
       GoogleFeedback GooglePartnerSetup Maps MarketUpdater NetworkLocation
       OneTimeInitializer QuickSearchBox Talk
       YouTube kickback soundback
       GoogleServicesFramework Vending "
  if [ "x$APPLY_EXTERNAL_BROWSER" == xyes ]; then
     SYS_APPS_REPLACE="$SYS_APPS_REPLACE 
       Browser HTMLViewer LatinIME SpareParts"
  fi
fi

for i in $SYS_APPS_REPLACE ; do
  echo "  replacing $i"
  $CP -p $REPLACE_DIR/app/*$i* $OUT_SYS/app/
done

if [ -d sys_app ]; then
  SYS_APPS_APPEND=`ls sys_app`
  for i in $SYS_APPS_APPEND ; do
    echo "  appending $i"
    if [ -f $OUT_SYS/app/$i ]; then
      $MV $OUT_SYS/app/$i $OUT_SYS/app-removed/
    fi
    $CP sys_app/$i $OUT_SYS/app/
    $CHOWN 0.0 $OUT_SYS/app/$i
    $CHMOD 644 $OUT_SYS/app/$i 
  done
fi

SYS_FONTS_REPLACE="DroidSansJapanese.ttf"

for i in $SYS_FONTS_REPLACE ; do
  echo "  replacing $i"
  $CP -p $REPLACE_DIR/fonts/*$i* $OUT_SYS/fonts/
done

if [ -d fonts ]; then
  SYS_FONTS_APPEND=`ls fonts`
  for i in $SYS_FONTS_APPEND ; do
    echo "  appending $i"
    $CP fonts/$i $OUT_SYS/fonts/
    $CHOWN 0.0 $OUT_SYS/fonts/$i
    $CHMOD 644 $OUT_SYS/fonts/$i 
  done
fi


  echo "  ==  appling turbo vm"
  SYS_LIB_REPLACE="libdvm.so"
  for i in $SYS_LIB_REPLACE ; do
    echo "  copying $i-turbo"
    $CP -p $OUT_SYS/lib/$i $OUT_SYS/lib/$i-org
    if [ -f $REPLACE_DIR/lib/$i-turbo ];then
      $CP -p $REPLACE_DIR/lib/$i-turbo $OUT_SYS/lib/$i-turbo
    else
      $CP -p $REPLACE_DIR/lib/$i $OUT_SYS/lib/$i-turbo
    fi
    if [ "x$APPLY_TURBO_VM" == "xyes" ];then
      echo "  replacing $i"
      $CP -p $REPLACE_DIR/lib/$i-turbo $OUT_SYS/lib/$i
    fi
  done
  echo "  ==  done turbo vm"

echo "  copying new userdata.img"
$CP -p $OUT_DAT_FILE $OUT_SYS/userdata.img

if [ "x$APPLY_ROOTKIT" == "xyes" ]; then
   echo "  ==  appling rootkit"
   if [ ! -f $OUT_SYS/xbin/su-bak ]; then
      echo "  copying su"
       $CP -p $OUT_SYS/xbin/su $OUT_SYS/xbin/su-bak
       cat root_app/su > $OUT_SYS/xbin/su
   fi
   echo "  copying Superuser.apk"
   $CP root_app/Superuser.apk $OUT_SYS/app/
   echo "  ==  done rootkit"
fi

if [ -f bin/busybox ]; then
	$CP bin/busybox $OUT_SYS/bin/unzip
	$CHOWN 0.0 $OUT_SYS/bin/unzip
	$CHMOD 755 $OUT_SYS/bin/unzip
fi
BUILD_PROGS=""
#BUILD_PROGS="mkbootfs mkbootimg split_nib split_bootimg"
for i in $BUILD_PROGS ; do
   if [ -f bin/$i ]; then
	$CP bin/$i $OUT_SYS/xbin
	$CHOWN 0.0 $OUT_SYS/xbin/$i
	$CHMOD 755 $OUT_SYS/xbin/$i
   fi
done

BUSYBOX_LINKS="awk basename bunzip2 bzcat bzip2 chroot cp cpio diff
        dirname du echo ed egrep expr getopt gunzip head hexdump hostname
        losetup mknod od patch printf pwd script sed sort swapoff
        swapon tail tar touch uname wc"
for i in $BUSYBOX_LINKS ; do 
    $LN -sf ../bin/busybox $OUT_SYS/xbin/$i
done
#        $LN -sf ../bin/busybox $i ; done )
#( cd $OUT_SYS/xbin ; for i in $BUSYBOX_LINKS ; do \
#        $LN -sf ../bin/busybox $i ; done )

echo "  filling zero"
$DD if=/dev/zero of=$OUT_SYS/123 bs=1024k 2> /dev/null
$RM -f $OUT_SYS/123

$UMOUNT $IN_SYS
$UMOUNT $OUT_SYS
$RMDIR $IN_SYS $OUT_SYS

if [ "x$SELF" == xyes ]; then
  for i in 0 1 2 3 4 5 6 7 ; do
	$LOSETUP -d /dev/loop$i 2> /dev/null
  done
fi

echo "done"
exit 0
